
; VCF-MW 12 6502 Badge Software
; Hardware design by Lee Hart
; Software design by Daryl Rictor
;
; This software is free to use and modify in any non-commercial application.
; Comericial use is prohibited without expressed, written permission from the author.
;
;------------------------------------------------------------------
; Bit-banged 9600,N,8,1 Serial I/O using 2MHz 65C02                |
;                                                                  |  
; Written by Daryl Rictor     5-18-2017                            |  
;------------------------------------------------------------------
;
; IRQ serial Input
;
;  2MHz clock / 9600 bits per seconds = 208.33 clocks per bit.
;
; State byte
; 9 = new byte, this is the start bit
; 1-8 = bytes 1 - 8 of data
; 0 = stop bit
; ff = reset to 9, store byte, and exit
;

Serbuf		=	$200		; 128 byte buffer
Bufin		=	$EA		; zp pointer
Bufout		=	$EB		; zp pointer
State		= 	$EC		; IRQ state machine pointer
Byte		= 	$ED		; temp received data byte
Dout 		= 	$E1		; ZP temp value
Port 		= 	$F0FE		; data port address ( G R Y LED's get cycled with this)
;LS		=	$80		; defined in font.asm based on bit position

;---------------------------------------------------------------------
; IRQ-based serial data input  (IRQ pin is used for RX data)
;
; Incoming data is stored in a 128 byte ring buffer.  If the buffer gets
; full, the last byte received will overwrite the last byte in the buffer.
; No error is generated for this condition.
;---------------------------------------------------------------------
IRQ		lda	State		; get current state
		cmp	#9		; start bit = 9
		beq	Startbit	; if 0, go to start of new byte	 
		pla			; discard acc
		pla			; discard IRQ status
		pla			; discard IRQ return address
		pla			;  "       "    "      "	
		clc			; set bit to 0
		ror	Byte		; save in received byte
		dec	State		; set state to next bit
		bmi	Stopbit		; yes
		ldx	$00		; 3 cycle delay
		ldx	#26		; init 1 bit time delay 26
		nop			; 2 cycles
		bra	Sb1		; 3 cycles get ready for next bit
Startbit	phx			; save X register
		dec	State		; set to bit 1 
		ldx	#52		; init 1.5 bit time delay 52
Sb1		dex			; do time delay loop
		bne	Sb1		; loop unitl x = 0
		cli			; enable IRQ in the center of the next bit
		nop			; give IRQ time to react
		nop			; safety delay
		nop			; safety delay
		sec			; set bit to 1 (no IRQ happened)
		ror	Byte		; save in received byte 
		ldx	#36		; 1 bit delay 36
		nop			; 2 cycles
		nop			; 2 cycles
Sb3		dec	State		; set state to next bit
		bpl	Sb1		; No, get ready for the next bit
Stopbit		ROL	Byte		; put data back and put stop bit in C
		bcc	Stoperr		; stop was 0, bad byte, ignore
		LDX	Bufin		; get pointer
		LDA	Byte		; get data
		STA	Serbuf,x	; safe in buffer
		txa			; save old pointer
		ina			; adjust pointer
		and	#$7F		; limit to 128 bytes
		cmp	Bufout		; check for full buffer		
		BNE	Stopcont	; not full
		txa			; full, don't advance pointer
Stopcont	sta	Bufin		; save pointer	
Stoperr		lda	#9		; state = start bit
		sta	State		; reset state for next byte
		plx			; restore X register
		pla			; restore Acc register
		rti			; go back to interrupted program			; go back to interrupted program

;---------------------------------------------------------------------
;  Bit-bang Serial data out using bit 7 of LED port
;---------------------------------------------------------------------
Output
Serial_Output
SEND		STA 	Dout		; save data to ZP
		pha			; Save Acc
        	phx			; save X
        	phy			; save Y
		LDX 	#8		; bit counter
		LDA 	Dout		; get port data
		AND 	#$FF-LS		; mask bit 7 (SER OUT)
		STA 	Port		; send Start bit
		NOP			;
		JMP	A1		; 5 cycle delay
A1		LDY 	#36		; 1 bit time delay
A2		DEY			;
		BNE  	A2		; delay loop
		LDA 	Dout		; get port data
		AND 	#$FF-LS		; mask bit 7 (SER OUT)
		ROR 	Dout		; get next data bit in C flag
		BCC  	A4		; if zero, we're set
		ORA 	#LS		; change bit 7 to 1
		JMP	A3		; delay
A4		NOP			; delay
		NOP			; delay
A3		STA 	Port		; write data bit to SER OUT
		DEX			; dec bit counter
		BNE  	A1		; next bit
		LDY	#38		; 
		JMP	A5		; 3 cyc delay
A5		DEY			;
		BNE	A5		;
		LDA 	Dout		; get port data
		ORA 	#LS		; set SER OUT bit to 1
		STA 	Port		; send stop bit
		ldy	#38		; delay for stop bit
A6		dey			;
		bne	A6		;
        	ply			; restore Y		
	        plx			; restore X
		pla			; restore A
		RTS			; done

;=================================================================
; System Functions
;=================================================================
;
; serial_init = initialize the serial IO
;
; serial_input = wait for receieved character and return in A
;
; serial_scan = scan for key, return C=0 if no key, C=1 for key with key in A
;
; serial_output = send a character out the serial port (located above)
;
; ----------------------------------------------------------------
; Initialization of serial IO
Serial_Init
		lda	#9		; state = start bit
		sta	State		; reset state for next byte
		STZ	Bufin		;
		stz	Bufout		;
		rts			; done

; ----------------------------------------------------------------
; Wait for Character to be received
;
Input_Chr
Serial_Input	
Sin1		jsr 	LED_scan	; refresh LED display
		lda	Bufout		; get buffer output
		cmp	Bufin		; compare with buffer input
		beq	Sin1		; nothing to get yet
		phx			; save X
		tax			; move pointer to X
		ina			; adjust pointer
		and	#$7F		; limit size to 128 bytes
		sta	Bufout		; save pointer
		lda	Serbuf,x	; get data
		plx			; restore X
		ora	#$00		; set flags for data byte
		rts			; done

; ----------------------------------------------------------------
; Scan for character (non-waiting)

Scan_Input
Serial_Scan	jsr 	LED_scan	; refresh LED display
Serial_ScanQ	lda	Bufout		; get buffer output
		cmp	Bufin		; compare with buffer input
		clc			; set no data flag
		beq	Sscan1		; nothing, return carry clear
		phx			; save X
		tax			; move pointer to X
		ina			; adjust pointer
		and	#$7F		; limit size to 128 bytes
		sta	Bufout		; save pointer
		lda	Serbuf,x	; get data
		plx			; restore X
		sec			; set data received flag
		ora	#$00		; set flags for data byte
Sscan1		rts			; done

